
// vbview_render.cpp
// vbview's render function
// Copyright (c) 1998-2010 by The VoxBo Development Team

// VoxBo is free software: you can redistribute it and/or modify it
// under the terms of the GNU General Public License as published by
// the Free Software Foundation, either version 3 of the License, or
// (at your option) any later version.
// 
// VoxBo is distributed in the hope that it will be useful, but
// WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// General Public License for more details.
// 
// You should have received a copy of the GNU General Public License
// along with VoxBo.  If not, see <http://www.gnu.org/licenses/>.
// 
// For general information on VoxBo, including the latest complete
// source code and binary distributions, manual, and associated files,
// see the VoxBo home page at: http://www.voxbo.org/
//
// original version written by Dan Kimberg

using namespace std;

#include "vbview.h"

// prototypes for purely local functions
inline QRgb alphablend(QRgb oldval,QRgb newval,int alpha);
void inline affine_add1x(VBMatrix &pos,VBMatrix &trans);
void inline affine_cr(VBMatrix &pos,VBMatrix &trans,int nx);

void
VBView::UniRender(vbrect rrect)
{
  // iterate through views, find ones whose 
  for (int i=0; i<(int)viewlist.size(); i++) {
    vbrect vrect(viewlist[i].xoff,viewlist[i].yoff,
                 viewlist[i].width,viewlist[i].height);
    vrect=vrect&rrect;  // intersection of view rect and render rect
    if (vrect)
      RenderSingle(viewlist[i]);
  }
}

void
VBView::UniRender(MyView &view)
{
  UniRenderView(view);
  return;
  if (!q_update)
    return;
  if (!layers.size())
    return;
  for (VBLayerI l=layers.begin(); l!=layers.end(); l++) {
    if (l==layers.begin() || (l->q_visible && l->alpha > 0))
      {};//UniRenderLayer(view,l);
  }
}

void
VBView::UniRenderView(MyView &view)
{
  if (!q_update)
    return;
  if (!layers.size())
    return;
  for (VBLayerI l=layers.begin(); l!=layers.end(); l++) {
    if (l==layers.begin() || (l->q_visible && l->alpha > 0))
      UniRenderLayer(view,l,vbrect(view.xoff,view.yoff,view.width,view.height));
  }
}

void
VBView::UniRenderLayer(MyView &view,VBLayerI layer,vbrect rrect)
{
  if (!q_update) return;
  if (!layer->q_visible) return;

  uint32 *p;
  VBMatrix traverse_x,traverse_cr,full;
  // calculate the transformation from window coordinates to this
  // layer's rendercube coordinates
  int pos=view.position;
  if (view.position==-1) {
    if (view.orient==MyView::vb_xy) pos=q_zslice;
    else if (view.orient==MyView::vb_yz) pos=q_xslice;
    else if (view.orient==MyView::vb_xz) pos=q_yslice;
  }
  layer->calcFullTransform(q_xscale,q_yscale,q_zscale,
                           view.xoff,view.yoff,view.width,
                           view.height,view.orient,pos,
                           q_fliph,q_flipv);
  full=layer->full; // for convenience
  // grab corner (0,0,0) voxel
  VBMatrix position(4,1);
  position.set(0,0,rrect.x);
  position.set(1,0,rrect.y);
  position.set(2,0,0);
  position.set(3,0,1);
  // now premultiply to get corresponding coord in rendercube
  // position^=layer->full;
  position^=full;
  traverse_x.init(4,1);
  traverse_x.set(0,0,full(0,0));
  traverse_x.set(1,0,full(1,0));
  traverse_x.set(2,0,full(2,0));
  traverse_x.set(3,0,0);
  traverse_cr.init(4,1);

  // the below replaced with the even more below, because we're
  // sometimes not drawing the entire width of the view
  //traverse_cr.set(0,0,full(0,1)-(full(0,0)*view.width));
  //traverse_cr.set(1,0,full(1,1)-(full(1,0)*view.width));
  //traverse_cr.set(2,0,full(2,1)-(full(2,0)*view.width));
  traverse_cr.set(0,0,full(0,1)-(full(0,0)*rrect.w));
  traverse_cr.set(1,0,full(1,1)-(full(1,0)*rrect.w));
  traverse_cr.set(2,0,full(2,1)-(full(2,0)*rrect.w));

  traverse_cr.set(3,0,0);
  // voila!  now, each time we increment x or y, we can do it without
  // matrix multiplication

  // let's figure out which values of i and j below should get
  // crosshairs.  find the coord in image space and map backwards.
  int xcross=-1,ycross=-1;
  if (q_crosshairs && layer->type==VBLayer::vb_struct) {
    VBMatrix tmpm(4,1),tmp2(4,4);
    invert(full,tmp2);
    tmpm.set(3,0,1);
    // FIXME note that we add 0.5 so that we can be roughly in the
    // middle of the voxel, not the corner
    if (view.orient==MyView::vb_xy) {
      tmpm.set(0,0,q_xslice+0.5);
      tmpm.set(1,0,q_yslice+0.5);
    }
    else if (view.orient==MyView::vb_yz) {
      tmpm.set(1,0,q_yslice+0.5);
      tmpm.set(2,0,q_zslice+0.5);
    }
    else if (view.orient==MyView::vb_xz) {
      tmpm.set(0,0,q_xslice+0.5);
      tmpm.set(2,0,q_zslice+0.5);
    }
    tmp2*=tmpm;
    xcross=(int)(tmp2(0,0));
    ycross=(int)(tmp2(1,0));
  }
  
  float xpos[3],xtx[3],xtcr[3];
  xpos[0]=position(0,0);
  xpos[1]=position(1,0);
  xpos[2]=position(2,0);
  xtx[0]=traverse_x(0,0);
  xtx[1]=traverse_x(1,0);
  xtx[2]=traverse_x(2,0);
  xtcr[0]=traverse_cr(0,0);
  xtcr[1]=traverse_cr(1,0);
  xtcr[2]=traverse_cr(2,0);
  QRgb newval,oldval;
  bool f_blend=(layer!=layers.begin());
  int32 xx,yy,zz;
  VBRegion myorigin;
  for (int j=rrect.y; j<rrect.y+rrect.h; j++) {
    p=(uint32 *)currentimage.scanLine(j);
    p+=(rrect.x);
    for (int i=rrect.x; i<rrect.x+rrect.w; i++) {
      oldval=*p;
      // we truncate these floating point coordinates, because when
      // you're magnifying a voxel, you don't start using the next
      // voxel's value until you cross the border.  we could've done
      // it by rounding, but it's cleaner this way.
      xx=(int)xpos[0];
      yy=(int)xpos[1];
      zz=(int)xpos[2];
      newval=layer->rendercube.getValue<int32>(xx,yy,zz);
      if (q_showorigin && xx==layer->rendercube.origin[0] && yy==layer->rendercube.origin[1] &&
          zz==layer->rendercube.origin[2]) {
        myorigin.add(i,j,0,0);
      }
      if (f_blend) {
        if (newval>>24) {
          newval=alphablend(oldval,newval,layer->alpha);
          *p=newval;
        }
      }
      else
        *p=newval;
      if (q_crosshairs && (j==ycross || i==xcross)) {
        *p=qRgb(255,0,0);
      }
      p++;
      xpos[0]+=xtx[0];
      xpos[1]+=xtx[1];
      xpos[2]+=xtx[2];
    }
    xpos[0]+=xtcr[0];
    xpos[1]+=xtcr[1];
    xpos[2]+=xtcr[2];
  }
  if (myorigin.size()) {
    double x,y,z;
    myorigin.GeometricCenter(x,y,z);
    // FIXME not 11, should adjust to magnification
    int32 x1=x-11,xn=x+11;
    int32 y1=y-11,yn=y+11;
    if (x1<view.xoff) x1=view.xoff;
    if (xn>view.xoff+view.width-1) xn=view.xoff+view.width-1;
    if (y1<view.yoff) y1=view.yoff;
    if (yn>view.yoff+view.height-1) yn=view.yoff+view.height-1;
    for (int j=y1; j<=yn; j++) {
      p=(uint32 *)currentimage.scanLine(j);
      p+=x1;

      for (int i=x1; i<=xn; i++) {
        double dist=sqrt((double)((i-x)*(i-x))
                         +(double)((j-y)*(j-y)));
        if (dist>9.5 && dist < 10.5)
          *p=qRgb(255,255,0);
        p++;
      }
    }
  }
  drawarea->update();
}

void inline
affine_add1x(VBMatrix &pos,VBMatrix &trans)
{
  pos.set(0,0,pos(0,0)+trans(0,0));
  pos.set(1,0,pos(1,0)+trans(1,0));
  pos.set(2,0,pos(2,0)+trans(2,0));
}

void inline
affine_cr(VBMatrix &pos,VBMatrix &trans,int nx)
{
  // first get back to the first voxel in this row
  pos.set(0,0,pos(0,0)-(trans(0,0)*nx));
  pos.set(1,0,pos(1,0)-(trans(1,0)*nx));
  pos.set(2,0,pos(2,0)-(trans(2,0)*nx));
  // now increment y
  pos.set(0,0,pos(0,0)+trans(0,1));
  pos.set(1,0,pos(1,0)+trans(1,1));
  pos.set(2,0,pos(2,0)+trans(2,1));
 
}

inline QRgb
alphablend(QRgb oldval,QRgb newval,int alpha)
{
  int oldfact=100-alpha;
  int newfact=alpha;
  int rr=(((oldval>>16)&0xff)*oldfact/100)+(((newval>>16)&0xff)*newfact/100);
  int gg=(((oldval>>8)&0xff)*oldfact/100)+(((newval>>8)&0xff)*newfact/100);
  int bb=(((oldval)&0xff)*oldfact/100)+(((newval)&0xff)*newfact/100);
  return qRgb(rr,gg,bb);
}





// colormask_affine() works for affine transforms.  for nonlinear
// transforms, which we don't do right now, we would have to check
// each voxel.  it takes as an argument a queue containing the pixel
// we clicked on in drawarea coordinates.  

// start with the view we clicked on.  call colormask_affine for that
// view, passing a region with the pixel we clicked on.  then map the
// mask coordinate onto each other view, finding the nearest
// coordinate in the plane of hte view (and a 3x3x3 region around it),
// and call colormask_affine for that.

int32
VBView::colormask_affine(vector<MyView>::iterator view,VBRegion queue,VBLayerI masklayer) {
  // note that the "visited" region is a region of screen coordinates,
  // not volume coordinates
  VBRegion visited;
  VBVoxel vv;
  //uint32 *p;
  // calculate the transformation from window coordinates to this
  // layer's rendercube coordinates
  int pos=view->position;
  if (view->position==-1) {
    if (view->orient==MyView::vb_xy) pos=q_zslice;
    else if (view->orient==MyView::vb_yz) pos=q_xslice;
    else if (view->orient==MyView::vb_xz) pos=q_yslice;
  }
  masklayer->calcFullTransform(q_xscale,q_yscale,q_zscale,
                               view->xoff,view->yoff,view->width,
                               view->height,view->orient,pos,
                               q_fliph,q_flipv);
  VBMatrix full=masklayer->full; // for convenience
  VBMatrix position(4,1);
  int xpos,ypos,zpos;
  // calculate the mask coordinate we're working on from the base
  // coord provided
  VBVoxel maskcoord;
  vv=queue.begin()->second;
  position.set(0,0,vv.x);
  position.set(1,0,vv.y);
  position.set(2,0,0);
  position.set(3,0,1);
  position^=full;    // premultiply to get layer coordinate
  maskcoord.x=(int)position(0,0);
  maskcoord.y=(int)position(1,0);
  maskcoord.z=(int)position(2,0);
  // if needed, push the old value on the current undo list
  if (!(masklayer->undo.front().contains(maskcoord.x,maskcoord.y,maskcoord.z)))
    masklayer->undo.front().add(maskcoord.x,maskcoord.y,maskcoord.z,
                                masklayer->cube.GetValue(maskcoord.x,maskcoord.y,maskcoord.z));
  // now write the new value
  masklayer->cube.SetValue(maskcoord.x,maskcoord.y,maskcoord.z,q_maskindex);
  masklayer->rendercube.SetValue(maskcoord.x,maskcoord.y,maskcoord.z,q_maskcolor);
  int32 xmin=vv.x,xmax=vv.x,ymin=vv.y,ymax=vv.y;
  while (queue.size()) {
    // pop the first voxel
    vv=queue.begin()->second;
    queue.remove(vv.x,vv.y,vv.z);
    visited.add(vv.x,vv.y,0,0);
    // see if it maps onto maskcoord
    position.set(0,0,vv.x);
    position.set(1,0,vv.y);
    position.set(2,0,0);
    position.set(3,0,1);
    position^=full;    // premultiply to get layer coordinate
    xpos=(int)position(0,0); ypos=(int)position(1,0); zpos=(int)position(2,0);
    // if this voxel didn't map onto the mask voxel, don't pursue it
    if (zpos!=maskcoord.z || xpos!=maskcoord.x || ypos!=maskcoord.y) continue;
    // FIXME do the actual coloring of vv.x,vv.y
    //p=(uint32 *)currentimage.scanLine(vv.y);
    //p+=vv.x;
    //*p=q_maskcolor;
    // add unvisited neighbors to queue
    if (vv.x<xmin) xmin=vv.x;
    if (vv.x>xmax) xmax=vv.x;
    if (vv.y<ymin) ymin=vv.y;
    if (vv.y>ymax) ymax=vv.y;
    for (int xx=vv.x-1; xx<=vv.x+1; xx++) {
      if (xx<0 || xx>view->width-1) continue;
      for (int yy=vv.y-1; yy<=vv.y+1; yy++) {
        if (yy<0 || yy>view->height-1) continue;
        if (visited.contains(xx,yy,0)) continue;
        // no choice but to...
        queue.add(xx,yy,0,0.0);
      }
    }
  }
  vbrect rr(xmin,ymin,xmax-xmin+1,ymax-ymin+1);
  for (VBLayerI l=layers.begin(); l!=layers.end(); l++) {
    if (l==layers.begin() || (l->q_visible && l->alpha > 0))
      UniRenderLayer(*view,l,rr);
  } 
  //drawarea->update();
  return 0;
}

